#!/usr/bin/env python3
# -*- coding: utf-8; -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import os

import fwk

import params.src.write as write

from Metafor.Mparams.surrogate import *
from Metafor.Mparams.sequential import evaluate
import Metafor.Mparams.TaskManager as tm

def master(model_parameters, surrogate_parameters, images_parameters):
    timers = fwk.Timers()
    timers['total'].start()
    timers['prepocessing'].start()


    dirname = 'master'
    if os.path.exists(dirname):
        shutil.rmtree(dirname)
    os.mkdir(dirname)
    os.chdir(dirname)

    f = write.init(surrogate_parameters['fname'])
    write.w_MPI_info(f, tm.name, tm.siz)

    write.w_list(f, model_parameters, 'Model parameters')
    write.w_list(f, surrogate_parameters, 'Surrogate parameters')
    ##
    timers['prepocessing'].stop()
    timers['training points pocessing'].start()
    if tm.siz == 1:
        sol, Z, W_sr, Image = evaluate(model_parameters, surrogate_parameters, images_parameters)
    else:
        master = tm.Master()
        sol, Z, W_sr, Image = master.start(model_parameters, surrogate_parameters, images_parameters)    

    timers['training points pocessing'].stop()
    timers['surrogate pocessing'].start()
    s_hat = surrogate_creation(sol, Z, W_sr, surrogate_parameters, Image, images_parameters)
    timers['surrogate pocessing'].stop()
    timers['total'].stop()

    write.w_vector(f, sol, 'Training points solutions') 
    write.w_vector(f, s_hat, 'Computed coefficients') 
    write.w_timers(f, timers)
    write.finish(f)

    return sol, Z, W_sr, s_hat

def worker():
    worker = tm.Worker(tm.rank)
    worker.start()

def AvailableComputedResults():
    out = True
    if not os.path.isfile('master/sol.json'):
        out = False
    if not os.path.isfile('master/Z.json'):
        out = False
    if not os.path.isfile('master/W_sr.json'):
        out = False
    return out

def LoadComputedResults():
    import codecs, json 

    obj_text = codecs.open('master/sol.json', 'r', encoding='utf-8').read()
    b_new = json.loads(obj_text)
    sol = np.array(b_new)

    obj_text = codecs.open('master/Z.json', 'r', encoding='utf-8').read()
    b_new = json.loads(obj_text)
    Z = np.array(b_new)

    obj_text = codecs.open('master/W_sr.json', 'r', encoding='utf-8').read()
    b_new = json.loads(obj_text)
    W_sr = np.array(b_new)

    return sol, Z, W_sr
