import numpy

from katoptron.eigenvalues import *
from scipy.sparse import eye
import matplotlib.pyplot as plt
import scipy

import katoptron.utilities as u


def spectral_radii_bound(A, omega_min=0., omega_max=2., n_omega=100):
    from scipy.sparse import tril
    from scipy.sparse import triu
    from scipy.sparse import eye
    from scipy.sparse import diags
    from scipy.sparse import issparse

    omegas = np.linspace(omega_min, omega_max, n_omega)
    rhos = np.ones((n_omega, ))

    Adiag = A.diagonal()

    invD = diags(1. / Adiag, 0)

    scaled_A = invD * A

    L = -tril(scaled_A, -1)
    U = -triu(scaled_A, 1)

    n = A.shape[0]

    #n = 100

    l_n = np.zeros((n, ))
    u_n = np.zeros((n, ))

    l_2n = np.zeros((n, n - 1))
    u_2n = np.zeros((n, n - 1))

    T_l_n = np.zeros((n, ))
    T_u_n = np.zeros((n, ))
    L_abs = np.abs(L)
    U_abs = np.abs(U)

    l_n = L_abs.sum(axis=1)
    u_n = U_abs.sum(axis=1)

    T_l_n = L_abs.max(axis=1)
    T_u_n = U_abs.max(axis=1)

    if np.max(l_n) > 1:
        print('error')
        return omegas, rhos
    if np.max(T_l_n) > 1:
        print('error')
        return omegas, rhos
    '''
    for i in range(0,n):
        print i
        print n
        tmp = L.getrow(i)
        T_l_n[i] = np.max(np.abs(tmp))
        tmp = U.getrow(i)
        T_l_n[i] = np.max(np.abs(tmp))
    '''

    for i in range(0, n):
        for j in range(0, i):
            u_2n[i, j] = u_n[i] + u_n[j]
            l_2n[i, j] = l_n[i] + l_n[j]
        for j in range(i + 1, n):
            u_2n[i, j - 1] = u_n[i] + u_n[j]
            l_2n[i, j - 1] = l_n[i] + l_n[j]

    for omega_i in range(0, n_omega):
        omega = omegas[omega_i]

        tmp_1 = np.divide(2 * np.abs(1 - omega) + omega * u_2n,
                          2. - omega * l_2n)

        max_1 = np.amax(tmp_1)

        tmp_2 = np.zeros((n, ))
        for i in range(0, n):
            scalar_1 = ((np.abs(1 - omega) + omega * u_n[i]) / (1 -
                                                                      omega * l_n[i]))
            scalar_2 = ((np.abs(1 - omega)) / (1 - omega * T_u_n[i]))
            scalar_3 = np.abs(1 - omega) + omega * u_n[i]
            scalar_4 = np.max([scalar_2, scalar_3])
            tmp_2[i] = np.min([scalar_1, scalar_4])

        max_2 = np.max(tmp_2)

        rhos[i] = np.max([max_1, max_2])

    return omegas, rhos


def is_Z_matrix(A):
    from scipy.sparse import diags
    Adiag = A.diagonal()

    D = diags(Adiag, 0)

    Aoff = A - D

    max_row = Aoff.max(axis=1)

    if np.max(max_row) <= 0:
        return True
    else:
        return False


def is_M_matrix(A):
    is_Z = is_Z_matrix(A)
    if not is_Z:
        return False
    is_spd = is_SPD(A)

    return is_spd


def spectral_radii(A, omegas, n_power=10):
    from scipy.sparse import tril
    from scipy.sparse import triu
    from scipy.sparse import eye
    from scipy.sparse import diags
    from scipy.sparse import issparse

    n_omega = len(omegas)
    rhos = np.ones((n_omega, ))

    E = -tril(A, -1)
    F = -triu(A, 1)
    invD = diags(1. / A.diagonal(), 0)
    n = A.shape[0]
    I = eye(n)

    for omega_i in range(0, n_omega):
        #print omega_i
        omega = omegas[omega_i]
        if omega == 0. or omega == 2.:
            rhos[omega_i] = 1.
        else:
            N = I - omega * invD * E
            M = (1. - omega) * I + omega * invD * F
            rhos[omega_i] = np.min([gen_power_method(M, N, n=n_power), 1.])

        #print rhos

    return rhos


def main():
    comm, rank, siz = u.import_MPI()

    post_process = True

    base_dir = '/Users/kimliegeois/Desktop/preload/katoptron_tests_1_cube_tet_block_5/1/'

    A = read_mm(base_dir+'A_mm_after_bc.txt', -1)
    A_u = read_mm(base_dir+'A_00_mm.txt', -1, True, True, False)
    A_T = read_mm(base_dir+'A_11_mm.txt', -1, True, True, True)

    B = jacobi_iteration_matrix(A)
    B_u = jacobi_iteration_matrix(get_symmetrical_part(A_u))
    B_T = jacobi_iteration_matrix(A_T)

    sp = spectral_radius(B)
    sp_u = spectral_radius(B_u)
    sp_T = spectral_radius(B_T)

    A_u_f = A_u.todense()
    index = np.array([3, 4, 5, 15, 16, 17, 18, 19, 20, 21, 22, 23])
    # => rho = 0.875 (sum off diag = 1.8904321, diag = 2.16049383) nu = 0.2
    index = np.array([3, 15, 18, 21])
    index = np.array([4, 16, 19, 22])  # => rho = 0.875
    # index = np.array([5,17,20,23]) #=> rho = 0.4285714285714285 nu = 0.2
    # => rho = .3035714285714295 (sum off diag = 3.2793209800000005, diag = 2.16049383) nu = 0.2
    index = np.array([3, 4, 15, 16, 18, 19, 21, 22])
    # if nu = 0 OK! rho(B_u) = 0.8125
    A_u_f_1 = A_u_f[index, :]
    A_u_f_2 = A_u_f_1[:, index]
    print(A_u_f_2)
    print(sp)
    print(sp_u)
    print(sp_T)

    B_u = jacobi_iteration_matrix(scipy.sparse.csr_matrix(A_u_f_2))

    print(B_u.todense())
    sp_u = spectral_radius(B_u)
    print(sp_u)


if __name__ == "__main__":
    main()
