# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# custom sources coded with python

import waves as w
import tbox
import math


class SinSource(w.Source):
    def __init__(self, msh, num_or_name, freq):
        w.Source.__init__(self, msh, num_or_name)
        self.freq = freq

    def apply(self, t, u):
        for n in self.nodes:  # necessite %nothread swig::SwigPyIterator::~SwigPyIterator() dans le .i
            u[n.row] = math.sin(2 * math.pi * self.freq * t)


class Src:
    def __init__(self, nod, freq):
        self.nod = nod
        self.freq = freq
#    def __del__(self):
#        print "[Src] I'm dying..."


class MultiSource(w.Source):
    def __init__(self, msh, num_or_name):
        w.Source.__init__(self, msh, num_or_name)

        self.srcs = []

    def apply(self, t, u):
        for s in self.srcs:  # necessite %nothread swig::SwigPyIterator::~SwigPyIterator() dans le .i
            u[s.nod.row] = math.sin(2 * math.pi * s.freq * t)

    def addSource(self, p, freq):
        dist = 1e10
        for n in self.nodes:
            v = n.pos - p
            v[2] = 0
            d = v.norm()
            if d < dist:
                dist = d
        for n in self.nodes:
            v = n.pos - p
            v[2] = 0
            d = v.norm()
            if abs(d - dist) < 1.0e-6:
                print("node source selected at ", n)
                print("dist", dist)
                print("nod=", n)
                self.srcs.append(Src(n, freq))

    def __del__(self):
        print("[MultiSource] I'm dying...")


class PulseSource(w.Source):
    def __init__(self, msh, num_or_name, freq, shift, width):
        w.Source.__init__(self, msh, num_or_name)
        self.freq = freq
        self.shift = shift
        self.width = width

    def apply(self, t, u):
        for n in self.nodes:
            shift = 0.3
            u[n.row] = math.sin(2 * math.pi * self.freq * t) * math.exp(
                -(t - self.shift) * (t - self.shift) * self.width)
